@@ -45,10 +45,14 @@ enum event
   VTY_TIMEOUT_RESET,
 #ifdef VTYSH
   VTYSH_SERV,
-  VTYSH_READ
+  VTYSH_READ,
+  VTYSH_WRITE
 #endif /* VTYSH */
 };
 
+/* Minimum size of output buffers; to be rounded up to multiple of pagesize. */
+#define VTY_OBUF_SIZE	4096
+
 static void vty_event (enum event, int, struct vty *);
 
 /* Extern host structure from command.c */
@@ -91,15 +95,19 @@ vty_out (struct vty *vty, const char *format, ...)
   int size = 1024;
   char buf[1024];
   char *p = NULL;
-  
-  va_start (args, format);
 
   if (vty_shell (vty))
-    vprintf (format, args);
+    {
+      va_start (args, format);
+      vprintf (format, args);
+      va_end (args);
+    }
   else
     {
       /* Try to write to initial buffer.  */
+      va_start (args, format);
       len = vsnprintf (buf, sizeof buf, format, args);
+      va_end (args);
 
       /* Initial buffer is not enough.  */
       if (len < 0 || len >= size)
@@ -115,7 +123,9 @@ vty_out (struct vty *vty, const char *format, ...)
 	      if (! p)
 		return -1;
 
+	      va_start (args, format);
 	      len = vsnprintf (p, size, format, args);
+	      va_end (args);
 
 	      if (len > -1 && len < size)
 		break;
@@ -127,30 +137,28 @@ vty_out (struct vty *vty, const char *format, ...)
 	p = buf;
 
       /* Pointer p must point out buffer. */
-      if (vty_shell_serv (vty))
-	write (vty->fd, (u_char *) p, len);
-      else
-	buffer_write (vty->obuf, (u_char *) p, len);
+      buffer_write (vty->obuf, (u_char *) p, len);
 
       /* If p is not different with buf, it is allocated buffer.  */
       if (p != buf)
 	XFREE (MTYPE_VTY_OUT_BUF, p);
     }
 
-  va_end (args);
-
   return len;
 }
 
-int
-vty_log_out (struct vty *vty, const char *proto_str, const char *format,
-	     va_list va)
+static int
+vty_log_out (struct vty *vty, const char *level, const char *proto_str,
+	     const char *format, va_list va)
 {
   int len;
   char buf[1024];
 
-  snprintf (buf, sizeof buf, "%s: ", proto_str);
-  write (vty->fd, buf, strlen (proto_str) + 2);
+  if (level)
+    snprintf (buf, sizeof buf, "%s: %s: ", level, proto_str);
+  else
+    snprintf (buf, sizeof buf, "%s: ", proto_str);
+  write (vty->fd, buf, strlen (buf));
 
   len = vsnprintf (buf, sizeof buf, format, va);
   if (len < 0)
@@ -221,7 +229,7 @@ vty_prompt (struct vty *vty)
 void
 vty_will_echo (struct vty *vty)
 {
-  char cmd[] = { IAC, WILL, TELOPT_ECHO, '\0' };
+  unsigned char cmd[] = { IAC, WILL, TELOPT_ECHO, '\0' };
   vty_out (vty, "%s", cmd);
 }
 
@@ -229,7 +237,7 @@ vty_will_echo (struct vty *vty)
 static void
 vty_will_suppress_go_ahead (struct vty *vty)
 {
-  char cmd[] = { IAC, WILL, TELOPT_SGA, '\0' };
+  unsigned char cmd[] = { IAC, WILL, TELOPT_SGA, '\0' };
   vty_out (vty, "%s", cmd);
 }
 
@@ -237,7 +245,7 @@ vty_will_suppress_go_ahead (struct vty *vty)
 static void
 vty_dont_linemode (struct vty *vty)
 {
-  char cmd[] = { IAC, DONT, TELOPT_LINEMODE, '\0' };
+  unsigned char cmd[] = { IAC, DONT, TELOPT_LINEMODE, '\0' };
   vty_out (vty, "%s", cmd);
 }
 
@@ -245,7 +253,7 @@ vty_dont_linemode (struct vty *vty)
 static void
 vty_do_window_size (struct vty *vty)
 {
-  char cmd[] = { IAC, DO, TELOPT_NAWS, '\0' };
+  unsigned char cmd[] = { IAC, DO, TELOPT_NAWS, '\0' };
   vty_out (vty, "%s", cmd);
 }
 
@@ -254,7 +262,7 @@ vty_do_window_size (struct vty *vty)
 static void
 vty_dont_lflow_ahead (struct vty *vty)
 {
-  char cmd[] = { IAC, DONT, TELOPT_LFLOW, '\0' };
+  unsigned char cmd[] = { IAC, DONT, TELOPT_LFLOW, '\0' };
   vty_out (vty, "%s", cmd);
 }
 #endif /* 0 */
@@ -264,8 +272,9 @@ struct vty *
 vty_new ()
 {
   struct vty *new = XCALLOC (MTYPE_VTY, sizeof (struct vty));
+  int pgsz = getpagesize();
 
-  new->obuf = (struct buffer *) buffer_new (100);
+  new->obuf = (struct buffer *) buffer_new ((((VTY_OBUF_SIZE-1)/pgsz)+1)*pgsz);
   new->buf = XCALLOC (MTYPE_VTY, VTY_BUFSIZ);
   new->max = VTY_BUFSIZ;
   new->sb_buffer = NULL;
@@ -352,7 +361,7 @@ vty_command (struct vty *vty, char *buf)
   if (vline == NULL)
     return CMD_SUCCESS;
 
-  ret = cmd_execute_command (vline, vty, NULL);
+  ret = cmd_execute_command (vline, vty, NULL, 0);
 
   if (ret != CMD_SUCCESS)
     switch (ret)
@@ -596,7 +605,7 @@ static void
 vty_down_level (struct vty *vty)
 {
   vty_out (vty, "%s", VTY_NEWLINE);
-  config_exit (NULL, vty, 0, NULL);
+  (*config_exit_cmd.func)(NULL, vty, 0, NULL);
   vty_prompt (vty);
   vty->cp = 0;
 }
@@ -848,9 +857,10 @@ vty_complete_command (struct vty *vty)
 
 void
 vty_describe_fold (struct vty *vty, int cmd_width,
-                 int desc_width, struct desc *desc)
+		   unsigned int desc_width, struct desc *desc)
 {
-  char *buf, *cmd, *p;
+  char *buf;
+  const char *cmd, *p;
   int pos;
 
   cmd = desc->cmd[0] == '.' ? desc->cmd + 1 : desc->cmd;
@@ -891,7 +901,7 @@ vty_describe_command (struct vty *vty)
   int ret;
   vector vline;
   vector describe;
-  int i, width, desc_width;
+  unsigned int i, width, desc_width;
   struct desc *desc, *desc_cr = NULL;
 
   vline = cmd_make_strvec (vty->buf);
@@ -934,7 +944,7 @@ vty_describe_command (struct vty *vty)
   for (i = 0; i < vector_max (describe); i++)
     if ((desc = vector_slot (describe, i)) != NULL)
       {
-	int len;
+	unsigned int len;
 
 	if (desc->cmd[0] == '\0')
 	  continue;
@@ -1203,9 +1213,7 @@ vty_execute (struct vty *vty)
   vty->cp = vty->length = 0;
   vty_clear_buf (vty);
 
-  if (vty->status != VTY_CLOSE 
-      && vty->status != VTY_START
-      && vty->status != VTY_CONTINUE)
+  if (vty->status != VTY_CLOSE )
     vty_prompt (vty);
 
   return ret;
@@ -1307,8 +1315,6 @@ vty_read (struct thread *thread)
 	    case CONTROL('C'):
 	    case 'q':
 	    case 'Q':
-	      if (vty->output_func)
-		(*vty->output_func) (vty, 1);
 	      vty_buffer_reset (vty);
 	      break;
 #if 0 /* More line does not work for "show ip bgp".  */
@@ -1318,8 +1324,6 @@ vty_read (struct thread *thread)
 	      break;
 #endif
 	    default:
-	      if (vty->output_func)
-		(*vty->output_func) (vty, 0);
 	      break;
 	    }
 	  continue;
@@ -1455,7 +1459,6 @@ static int
 vty_flush (struct thread *thread)
 {
   int erase;
-  int dont_more;
   int vty_sock = THREAD_FD (thread);
   struct vty *vty = THREAD_ARG (thread);
   vty->t_write = NULL;
@@ -1469,55 +1472,6 @@ vty_flush (struct thread *thread)
       }
 
   /* Function execution continue. */
-  if (vty->status == VTY_START || vty->status == VTY_CONTINUE)
-    {
-      if (vty->status == VTY_CONTINUE)
-	erase = 1;
-      else
-	erase = 0;
-
-      if (vty->output_func == NULL)
-	dont_more = 1;
-      else
-	dont_more = 0;
-
-      if (vty->lines == 0)
-	{
-	  erase = 0;
-	  dont_more = 1;
-	}
-
-      buffer_flush_vty_all (vty->obuf, vty->fd, erase, dont_more);
-
-      if (vty->status == VTY_CLOSE)
-	{
-	  vty_close (vty);
-	  return 0;
-	}
-
-      if (vty->output_func == NULL)
-	{
-	  vty->status = VTY_NORMAL;
-	  vty_prompt (vty);
-	  vty_event (VTY_WRITE, vty_sock, vty);
-	}
-      else
-	vty->status = VTY_MORE;
-
-      if (vty->lines == 0)
-	{
-	  if (vty->output_func == NULL)
-	    vty_event (VTY_READ, vty_sock, vty);
-	  else
-	    {
-	      if (vty->output_func)
-		(*vty->output_func) (vty, 0);
-	      vty_event (VTY_WRITE, vty_sock, vty);
-	    }
-	}
-    }
-  else
-    {
       if (vty->status == VTY_MORE || vty->status == VTY_MORELINE)
 	erase = 1;
       else
@@ -1551,7 +1505,6 @@ vty_flush (struct thread *thread)
 	  if (vty->lines == 0)
 	    vty_event (VTY_WRITE, vty_sock, vty);
 	}
-    }
 
   return 0;
 }
@@ -1652,7 +1605,7 @@ vty_accept (struct thread *thread)
   vty_sock = sockunion_accept (accept_sock, &su);
   if (vty_sock < 0)
     {
-      zlog_warn ("can't accept vty socket : %s", strerror (errno));
+      zlog_warn ("can't accept vty socket : %s", safe_strerror (errno));
       return -1;
     }
 
@@ -1709,7 +1662,7 @@ vty_accept (struct thread *thread)
 		    (char *) &on, sizeof (on));
   if (ret < 0)
     zlog (NULL, LOG_INFO, "can't set sockopt to vty_sock : %s", 
-	  strerror (errno));
+	  safe_strerror (errno));
 
   vty = vty_create (vty_sock, &su);
 
@@ -1812,7 +1765,7 @@ vty_serv_sock_family (const char* addr, unsigned short port, int family)
 	naddr=NULL;
 	break;
       case 0:
-        zlog_err("error translating address %s: %s",addr,strerror(errno));
+        zlog_err("error translating address %s: %s",addr,safe_strerror(errno));
 	naddr=NULL;
     }
 
@@ -1853,7 +1806,7 @@ vty_serv_sock_family (const char* addr, unsigned short port, int family)
 
 /* VTY shell UNIX domain socket. */
 void
-vty_serv_un (char *path)
+vty_serv_un (const char *path)
 {
   int ret;
   int sock, len;
@@ -1911,7 +1864,7 @@ vty_serv_un (char *path)
       if ( chown (path, -1, ids.gid_vty) )
         {
           zlog_err ("vty_serv_un: could chown socket, %s",
-                     strerror (errno) );
+                     safe_strerror (errno) );
         }
     }
 
@@ -1926,6 +1879,7 @@ vtysh_accept (struct thread *thread)
   int accept_sock;
   int sock;
   int client_len;
+  int flags;
   struct sockaddr_un client;
   struct vty *vty;
   
@@ -1936,14 +1890,25 @@ vtysh_accept (struct thread *thread)
   memset (&client, 0, sizeof (struct sockaddr_un));
   client_len = sizeof (struct sockaddr_un);
 
-  sock = accept (accept_sock, (struct sockaddr *) &client, &client_len);
+  sock = accept (accept_sock, (struct sockaddr *) &client,
+		 (socklen_t *) &client_len);
 
   if (sock < 0)
     {
-      zlog_warn ("can't accept vty socket : %s", strerror (errno));
+      zlog_warn ("can't accept vty socket : %s", safe_strerror (errno));
       return -1;
     }
 
+  /* set to non-blocking*/
+  if ( ((flags = fcntl (sock, F_GETFL)) == -1)
+      || (fcntl (sock, F_SETFL, flags|O_NONBLOCK) == -1) )
+    {
+      zlog_warn ("vtysh_accept: could not set vty socket to non-blocking,"
+                 " %s, closing", safe_strerror (errno));
+      close (sock);
+      return -1;
+    }
+  
 #ifdef VTYSH_DEBUG
   printf ("VTY shell accept\n");
 #endif /* VTYSH_DEBUG */
@@ -2001,17 +1966,31 @@ vtysh_read (struct thread *thread)
 #endif /* VTYSH_DEBUG */
 
   header[3] = ret;
-  write (vty->fd, header, 4);
+  buffer_write(vty->obuf, header, 4);
 
+  if (!vty->t_write && buffer_flush_available(vty->obuf, vty->fd))
+    vty_event (VTYSH_WRITE, vty->fd, vty);
   vty_event (VTYSH_READ, sock, vty);
 
   return 0;
 }
+
+static int
+vtysh_write (struct thread *thread)
+{
+  struct vty *vty = THREAD_ARG (thread);
+
+  vty->t_write = NULL;
+  if (buffer_flush_available(vty->obuf, vty->fd))
+    vty_event (VTYSH_WRITE, vty->fd, vty);
+  return 0;
+}
+
 #endif /* VTYSH */
 
 /* Determine address family to bind. */
 void
-vty_serv_sock (const char *addr, unsigned short port, char *path)
+vty_serv_sock (const char *addr, unsigned short port, const char *path)
 {
   /* If port is set to 0, do not listen on TCP/IP at all! */
   if (port)
@@ -2047,8 +2026,6 @@ vty_close (struct vty *vty)
     thread_cancel (vty->t_write);
   if (vty->t_timeout)
     thread_cancel (vty->t_timeout);
-  if (vty->t_output)
-    thread_cancel (vty->t_output);
 
   /* Flush buffer. */
   if (! buffer_empty (vty->obuf))
@@ -2202,14 +2179,13 @@ vty_use_backup_config (char *fullpath)
   
   free (fullpath_sav);
   free (fullpath_tmp);
-  return fopen (fullpath, "r");
+  return ret;
 }
 
 /* Read up configuration file from file_name. */
 void
 vty_read_config (char *config_file,
-		 char *config_current_dir,
-		 char *config_default_dir)
+                 char *config_default_dir)
 {
   char cwd[MAXPATHLEN];
   FILE *confp = NULL;
@@ -2219,101 +2195,78 @@ vty_read_config (char *config_file,
   if (config_file != NULL)
     {
       if (! IS_DIRECTORY_SEP (config_file[0]))
-	{
-	  getcwd (cwd, MAXPATHLEN);
-	  fullpath = XMALLOC (MTYPE_TMP, 
-			      strlen (cwd) + strlen (config_file) + 2);
-	  sprintf (fullpath, "%s/%s", cwd, config_file);
-	}
+        {
+          getcwd (cwd, MAXPATHLEN);
+          fullpath = XMALLOC (MTYPE_TMP, 
+ 			      strlen (cwd) + strlen (config_file) + 2);
+          sprintf (fullpath, "%s/%s", cwd, config_file);
+        }
       else
-	fullpath = config_file;
+        fullpath = config_file;
 
       confp = fopen (fullpath, "r");
 
       if (confp == NULL)
-	{
-	  confp = vty_use_backup_config (fullpath);
-	  if (confp)
-	    fprintf (stderr, "WARNING: using backup configuration file!\n");
-	  else
-	    {
-	      fprintf (stderr, "can't open configuration file [%s]\n", 
-		       config_file);
-	      exit(1);
-	    }
-	}
+        {
+          confp = vty_use_backup_config (fullpath);
+          if (confp)
+            fprintf (stderr, "WARNING: using backup configuration file!\n");
+          else
+            {
+              fprintf (stderr, "can't open configuration file [%s]\n", 
+  	            config_file);
+              exit(1);
+            }
+        }
     }
   else
     {
-      /* Relative path configuration file open. */
-      if (config_current_dir)
-	{
-	  confp = fopen (config_current_dir, "r");
-	  if (confp == NULL)
-	    {
-	      confp = vty_use_backup_config (config_current_dir);
-	      if (confp)
-		fprintf (stderr, "WARNING: using backup configuration file!\n");
-	    }
-	}
-
-      /* If there is no relative path exists, open system default file. */
-      if (confp == NULL)
-	{
 #ifdef VTYSH
-	  int ret;
-	  struct stat conf_stat;
+      int ret;
+      struct stat conf_stat;
 
-	  /* !!!!PLEASE LEAVE!!!!
-	     This is NEEDED for use with vtysh -b, or else you can get
-	     a real configuration food fight with a lot garbage in the
-	     merged configuration file it creates coming from the per
-	     daemon configuration files.  This also allows the daemons
-	     to start if there default configuration file is not
-	     present or ignore them, as needed when using vtysh -b to
-	     configure the daemons at boot - MAG */
+      /* !!!!PLEASE LEAVE!!!!
+       * This is NEEDED for use with vtysh -b, or else you can get
+       * a real configuration food fight with a lot garbage in the
+       * merged configuration file it creates coming from the per
+       * daemon configuration files.  This also allows the daemons
+       * to start if there default configuration file is not
+       * present or ignore them, as needed when using vtysh -b to
+       * configure the daemons at boot - MAG
+       */
 
-	  /* Stat for vtysh Zebra.conf, if found startup and wait for
-	     boot configuration */
+      /* Stat for vtysh Zebra.conf, if found startup and wait for
+       * boot configuration
+       */
 
-	  if ( strstr(config_default_dir, "vtysh") == NULL)
-	    {
-	      ret = stat (integrate_default, &conf_stat);
-	      if (ret >= 0)
-		{
-		  return;
-		}
-	    }
+      if ( strstr(config_default_dir, "vtysh") == NULL)
+        {
+          ret = stat (integrate_default, &conf_stat);
+          if (ret >= 0)
+            return;
+        }
 #endif /* VTYSH */
 
-	  confp = fopen (config_default_dir, "r");
-	  if (confp == NULL)
-	    {
-	      confp = vty_use_backup_config (config_default_dir);
-	      if (confp)
-		{
-		  fprintf (stderr, "WARNING: using backup configuration file!\n");
-		  fullpath = config_default_dir;
-		}
-	      else
-		{
-		  fprintf (stderr, "can't open configuration file [%s]\n",
-			   config_default_dir);
-		  exit (1);
-		}
-	    }      
-	  else
-	    fullpath = config_default_dir;
-	}
+      confp = fopen (config_default_dir, "r");
+      if (confp == NULL)
+        {
+          confp = vty_use_backup_config (config_default_dir);
+          if (confp)
+            {
+              fprintf (stderr, "WARNING: using backup configuration file!\n");
+              fullpath = config_default_dir;
+            }
+          else
+            {
+              fprintf (stderr, "can't open configuration file [%s]\n",
+  		                 config_default_dir);
+  	          exit (1);
+  	        }
+        }      
       else
-	{
-	  /* Rleative path configuration file. */
-	  getcwd (cwd, MAXPATHLEN);
-	  fullpath = XMALLOC (MTYPE_TMP, 
-			      strlen (cwd) + strlen (config_current_dir) + 2);
-	  sprintf (fullpath, "%s/%s", cwd, config_current_dir);
-	}  
-    }  
+        fullpath = config_default_dir;
+    }
+
   vty_read_file (confp);
 
   fclose (confp);
@@ -2323,15 +2276,39 @@ vty_read_config (char *config_file,
 
 /* Small utility function which output log to the VTY. */
 void
-vty_log (const char *proto_str, const char *format, va_list va)
+vty_log (const char *level, const char *proto_str,
+	 const char *format, va_list va)
 {
-  int i;
+  unsigned int i;
   struct vty *vty;
 
   for (i = 0; i < vector_max (vtyvec); i++)
     if ((vty = vector_slot (vtyvec, i)) != NULL)
       if (vty->monitor)
-	vty_log_out (vty, proto_str, format, va);
+	{
+	  va_list ac;
+	  va_copy(ac, va);
+	  vty_log_out (vty, level, proto_str, format, ac);
+	  va_end(ac);
+	}
+}
+
+/* Async-signal-safe version of vty_log for fixed strings. */
+void
+vty_log_fixed (const char *buf, size_t len)
+{
+  unsigned int i;
+
+  for (i = 0; i < vector_max (vtyvec); i++)
+    {
+      struct vty *vty;
+      if ((vty = vector_slot (vtyvec, i)) != NULL)
+	if (vty->monitor)
+	  {
+	    write(vty->fd, buf, len);
+	    write(vty->fd, "\r\n", 2);
+	  }
+    }
 }
 
 int
@@ -2375,7 +2352,10 @@ vty_event (enum event event, int sock, struct vty *vty)
       thread_add_read (master, vtysh_accept, vty, sock);
       break;
     case VTYSH_READ:
-      thread_add_read (master, vtysh_read, vty, sock);
+      vty->t_read = thread_add_read (master, vtysh_read, vty, sock);
+      break;
+    case VTYSH_WRITE:
+      vty->t_write = thread_add_write (master, vtysh_write, vty, sock);
       break;
 #endif /* VTYSH */
     case VTY_READ:
@@ -2414,7 +2394,7 @@ DEFUN (config_who,
        "who",
        "Display who is on vty\n")
 {
-  int i;
+  unsigned int i;
   struct vty *v;
 
   for (i = 0; i < vector_max (vtyvec); i++)
@@ -2438,7 +2418,7 @@ DEFUN (line_vty,
 
 /* Set time out value. */
 int
-exec_timeout (struct vty *vty, char *min_str, char *sec_str)
+exec_timeout (struct vty *vty, const char *min_str, const char *sec_str)
 {
   unsigned long timeout = 0;
 
@@ -2687,13 +2667,14 @@ struct cmd_node vty_node =
 {
   VTY_NODE,
   "%s(config-line)# ",
+  1,
 };
 
 /* Reset all VTY status. */
 void
 vty_reset ()
 {
-  int i;
+  unsigned int i;
   struct vty *vty;
   struct thread *vty_serv_thread;
 
@@ -2733,7 +2714,7 @@ vty_reset ()
 void
 vty_finish ()
 {
-  int i;
+  unsigned int i;
   struct vty *vty;
   struct thread *vty_serv_thread;
 
