@@ -97,6 +97,9 @@ ospf6_get_lsa_handler (u_int16_t type)
   else
     handler = vector_slot (ospf6_lsa_handler_vector, index);
 
+  if (handler == NULL)
+    handler = &unknown_handler;
+
   return handler;
 }
 
@@ -179,7 +182,7 @@ ospf6_lsa_age_set (struct ospf6_lsa *lsa)
 
   if (gettimeofday (&now, (struct timezone *)NULL) < 0)
     zlog_warn ("LSA: gettimeofday failed, may fail LSA AGEs: %s",
-               strerror (errno));
+               safe_strerror (errno));
 
   lsa->birth.tv_sec = now.tv_sec - ntohs (lsa->header->age);
   lsa->birth.tv_usec = now.tv_usec;
@@ -202,7 +205,7 @@ ospf6_lsa_age_current (struct ospf6_lsa *lsa)
   /* current time */
   if (gettimeofday (&now, (struct timezone *)NULL) < 0)
     zlog_warn ("LSA: gettimeofday failed, may fail LSA AGEs: %s",
-               strerror (errno));
+               safe_strerror (errno));
 
   /* calculate age */
   ulage = now.tv_sec - lsa->birth.tv_sec;
@@ -231,7 +234,7 @@ ospf6_lsa_premature_aging (struct ospf6_lsa *lsa)
 {
   /* log */
   if (IS_OSPF6_DEBUG_LSA_TYPE (lsa->header->type))
-    zlog_info ("LSA: Premature aging: %s", lsa->name);
+    zlog_debug ("LSA: Premature aging: %s", lsa->name);
 
   THREAD_OFF (lsa->expire);
   THREAD_OFF (lsa->refresh);
@@ -312,11 +315,11 @@ ospf6_lsa_header_print_raw (struct ospf6_lsa_header *header)
   inet_ntop (AF_INET, &header->id, id, sizeof (id));
   inet_ntop (AF_INET, &header->adv_router, adv_router,
              sizeof (adv_router));
-  zlog_info ("    [%s Id:%s Adv:%s]",
-             ospf6_lstype_name (header->type), id, adv_router);
-  zlog_info ("    Age: %4hu SeqNum: %#08lx Cksum: %04hx Len: %d",
-             ntohs (header->age), (u_long) ntohl (header->seqnum),
-             ntohs (header->checksum), ntohs (header->length));
+  zlog_debug ("    [%s Id:%s Adv:%s]",
+	      ospf6_lstype_name (header->type), id, adv_router);
+  zlog_debug ("    Age: %4hu SeqNum: %#08lx Cksum: %04hx Len: %d",
+	      ntohs (header->age), (u_long) ntohl (header->seqnum),
+	      ntohs (header->checksum), ntohs (header->length));
 }
 
 void
@@ -584,7 +587,7 @@ ospf6_lsa_expire (struct thread *thread)
 
   if (IS_OSPF6_DEBUG_LSA_TYPE (lsa->header->type))
     {
-      zlog_info ("LSA Expire:");
+      zlog_debug ("LSA Expire:");
       ospf6_lsa_header_print (lsa);
     }
 
@@ -621,7 +624,7 @@ ospf6_lsa_refresh (struct thread *thread)
   if (self == NULL)
     {
       if (IS_OSPF6_DEBUG_LSA_TYPE (old->header->type))
-        zlog_info ("Refresh: could not find self LSA, flush %s", old->name);
+        zlog_debug ("Refresh: could not find self LSA, flush %s", old->name);
       ospf6_lsa_premature_aging (old);
       return 0;
     }
@@ -643,7 +646,7 @@ ospf6_lsa_refresh (struct thread *thread)
 
   if (IS_OSPF6_DEBUG_LSA_TYPE (new->header->type))
     {
-      zlog_info ("LSA Refresh:");
+      zlog_debug ("LSA Refresh:");
       ospf6_lsa_header_print (new);
     }
 
