@@ -74,6 +74,8 @@ struct option longopts[] =
   { "pid_file",    required_argument, NULL, 'i'},
   { "vty_addr",    required_argument, NULL, 'A'},
   { "vty_port",    required_argument, NULL, 'P'},
+  { "user",        required_argument, NULL, 'u'},
+  { "group",       required_argument, NULL, 'g'},
   { "version",     no_argument,       NULL, 'v'},
   { "help",        no_argument,       NULL, 'h'},
   { 0 }
@@ -109,6 +111,8 @@ Daemon which manages OSPF version 3.\n\n\
 -i, --pid_file     Set process identifier file name\n\
 -A, --vty_addr     Set vty's bind address\n\
 -P, --vty_port     Set vty's port number\n\
+-u, --user         User to run as\n\
+-g, --group        Group to run as\n\
 -v, --version      Print program version\n\
 -h, --help         Display this help and exit\n\
 \n\
@@ -129,7 +133,7 @@ sighup (void)
 void
 sigint (void)
 {
-  zlog_info ("SIGINT received");
+  zlog_notice ("Terminating on signal SIGINT");
   exit (0);
 }
 
@@ -137,7 +141,7 @@ sigint (void)
 void
 sigterm (void)
 {
-  zlog_info ("SIGTERM received");
+  zlog_notice ("Terminating on signal SIGTERM");
   exit (0);
 }
 
@@ -180,7 +184,6 @@ main (int argc, char *argv[], char *envp[])
   int vty_port = 0;
   char *config_file = NULL;
   struct thread thread;
-  int flag;
 
   /* Set umask before anything for security */
   umask (0027);
@@ -191,7 +194,7 @@ main (int argc, char *argv[], char *envp[])
   /* Command line argument treatment. */
   while (1) 
     {
-      opt = getopt_long (argc, argv, "df:i:hp:A:P:u:v", longopts, 0);
+      opt = getopt_long (argc, argv, "df:i:hp:A:P:u:g:v", longopts, 0);
     
       if (opt == EOF)
         break;
@@ -224,8 +227,11 @@ main (int argc, char *argv[], char *envp[])
           vty_port = (vty_port ? vty_port : OSPF6_VTY_PORT);
           break;
         case 'u':
-          ospf6d_privs.user = ospf6d_privs.group = optarg;
+          ospf6d_privs.user = optarg;
           break;
+	case 'g':
+	  ospf6d_privs.group = optarg;
+	  break;
         case 'v':
           print_version (progname);
           exit (0);
@@ -243,12 +249,7 @@ main (int argc, char *argv[], char *envp[])
   master = thread_master_create ();
 
   /* Initializations. */
-  if (! daemon_mode)
-    flag = ZLOG_STDOUT;
-  else
-    flag = 0;
-
-  zlog_default = openzlog (progname, flag, ZLOG_OSPF6,
+  zlog_default = openzlog (progname, ZLOG_OSPF6,
                            LOG_CONS|LOG_NDELAY|LOG_PID,
                            LOG_DAEMON);
   zprivs_init (&ospf6d_privs);
@@ -274,19 +275,16 @@ main (int argc, char *argv[], char *envp[])
     daemon (0, 0);
 
   /* pid file create */
-#if 0
-  pid_output_lock (pid_file);
-#else
   pid_output (pid_file);
-#endif
 
   /* Make ospf6 vty socket. */
-  vty_serv_sock (vty_addr,
-                 vty_port ? vty_port : OSPF6_VTY_PORT, OSPF6_VTYSH_PATH);
+  if (!vty_port)
+    vty_port = OSPF6_VTY_PORT;
+  vty_serv_sock (vty_addr, vty_port, OSPF6_VTYSH_PATH);
 
   /* Print start message */
-  zlog_notice ("OSPF6d (Quagga-%s ospf6d-%s) starts",
-               QUAGGA_VERSION, OSPF6_DAEMON_VERSION);
+  zlog_notice ("OSPF6d (Quagga-%s ospf6d-%s) starts: vty@%d",
+               QUAGGA_VERSION, OSPF6_DAEMON_VERSION,vty_port);
 
   /* Start finite state machine, here we go! */
   while (thread_fetch (master, &thread))
