@@ -192,6 +192,17 @@ ospf_fifo_free (struct ospf_fifo *fifo)
 void
 ospf_packet_add (struct ospf_interface *oi, struct ospf_packet *op)
 {
+  if (!oi->obuf)
+    {
+      zlog_err("ospf_packet_add(interface %s in state %d [%s], packet type %s, "
+	       "destination %s) called with NULL obuf, ignoring "
+	       "(please report this bug)!\n",
+	       IF_NAME(oi), oi->state, LOOKUP (ospf_ism_state_msg, oi->state),
+	       ospf_packet_type_str[stream_getc_from(op->s, 1)],
+	       inet_ntoa (op->dst));
+      return;
+    }
+
   /* Add packet to end of queue. */
   ospf_fifo_push (oi->obuf, op);
 
@@ -241,16 +252,25 @@ ospf_packet_dup (struct ospf_packet *op)
   return new;
 }
 
+/* XXX inline */
+unsigned int
+ospf_packet_authspace (struct ospf_interface *oi)
+{
+  int auth = 0;
+
+  if ( ospf_auth_type (oi) == OSPF_AUTH_CRYPTOGRAPHIC)
+    auth = OSPF_AUTH_MD5_SIZE;
+
+  return auth;
+}
+
 unsigned int
 ospf_packet_max (struct ospf_interface *oi)
 {
   int max;
 
-  if ( ospf_auth_type (oi) == OSPF_AUTH_CRYPTOGRAPHIC)
-    max = oi->ifp->mtu - OSPF_AUTH_MD5_SIZE;
-  else
-    max = oi->ifp->mtu;
-  
+  max = oi->ifp->mtu - ospf_packet_authspace(oi);
+
   max -= (OSPF_HEADER_SIZE + sizeof (struct ip));
 
   return max;
@@ -539,18 +559,18 @@ ospf_write_frags (int fd, struct ospf_packet *op, struct ip *iph,
                    iph->ip_id,
                    iph->ip_off,
                    iph->ip_len,
-                   strerror (errno));
+                   safe_strerror (errno));
       
       if (IS_DEBUG_OSPF_PACKET (type - 1, SEND))
         {
-          zlog_info ("ospf_write_frags: sent id %d, off %d, len %d to %s\n",
+          zlog_debug ("ospf_write_frags: sent id %d, off %d, len %d to %s\n",
                      iph->ip_id, iph->ip_off, iph->ip_len,
                      inet_ntoa (iph->ip_dst));
           if (IS_DEBUG_OSPF_PACKET (type - 1, DETAIL))
             {
-              zlog_info ("-----------------IP Header Dump----------------------");
+              zlog_debug ("-----------------IP Header Dump----------------------");
               ospf_ip_header_dump (iph);
-              zlog_info ("-----------------------------------------------------");
+              zlog_debug ("-----------------------------------------------------");
             }
         }
       
@@ -690,25 +710,25 @@ ospf_write (struct thread *thread)
   
   if (ret < 0)
     zlog_warn ("*** sendmsg in ospf_write to %s failed with %s",
-      inet_ntoa (iph.ip_dst), strerror (errno));
+      inet_ntoa (iph.ip_dst), safe_strerror (errno));
 
   /* Show debug sending packet. */
   if (IS_DEBUG_OSPF_PACKET (type - 1, SEND))
     {
       if (IS_DEBUG_OSPF_PACKET (type - 1, DETAIL))
 	{
-	  zlog_info ("-----------------------------------------------------");
+	  zlog_debug ("-----------------------------------------------------");
 	  ospf_ip_header_dump (&iph);
 	  stream_set_getp (op->s, 0);
 	  ospf_packet_dump (op->s);
 	}
 
-      zlog_info ("%s sent to [%s] via [%s].",
+      zlog_debug ("%s sent to [%s] via [%s].",
 		 ospf_packet_type_str[type], inet_ntoa (op->dst),
 		 IF_NAME (oi));
 
       if (IS_DEBUG_OSPF_PACKET (type - 1, DETAIL))
-	zlog_info ("-----------------------------------------------------");
+	zlog_debug ("-----------------------------------------------------");
     }
 
   /* Now delete packet from queue. */
@@ -748,7 +768,7 @@ ospf_hello (struct ip *iph, struct ospf_header *ospfh,
     {
       if (IS_DEBUG_OSPF_PACKET (ospfh->type - 1, RECV))
         {
-          zlog_info ("ospf_header[%s/%s]: selforiginated, "
+          zlog_debug ("ospf_header[%s/%s]: selforiginated, "
                      "dropping.",
                      ospf_packet_type_str[ospfh->type],
                      inet_ntoa (iph->ip_src));
@@ -796,7 +816,7 @@ ospf_hello (struct ip *iph, struct ospf_header *ospfh,
     }
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("Packet %s [Hello:RECV]: Options %s",
+    zlog_debug ("Packet %s [Hello:RECV]: Options %s",
 	       inet_ntoa (ospfh->router_id),
 	       ospf_options_dump (hello->options));
 
@@ -846,7 +866,7 @@ ospf_hello (struct ip *iph, struct ospf_header *ospfh,
 	  return;
 	}
       if (IS_DEBUG_OSPF_NSSA)
-        zlog_info ("NSSA-Hello:RECV:Packet from %s:", inet_ntoa(ospfh->router_id));
+        zlog_debug ("NSSA-Hello:RECV:Packet from %s:", inet_ntoa(ospfh->router_id));
     }
   else    
     /* The setting of the E-bit found in the Hello Packet's Options
@@ -857,8 +877,8 @@ ospf_hello (struct ip *iph, struct ospf_header *ospfh,
     if (CHECK_FLAG (OPTIONS (oi), OSPF_OPTION_E) !=
 	CHECK_FLAG (hello->options, OSPF_OPTION_E))
       {
-	zlog_warn ("Packet[Hello:RECV]: my options: %x, his options %x",
-		   OPTIONS (oi), hello->options);
+	zlog_warn ("Packet %s [Hello:RECV]: my options: %x, his options %x",
+		   inet_ntoa(ospfh->router_id), OPTIONS (oi), hello->options);
 	return;
       }
   
@@ -978,7 +998,7 @@ ospf_db_desc_proc (struct stream *s, struct ospf_interface *oi,
       /* Unknown LS type. */
       if (lsah->type < OSPF_MIN_LSA || lsah->type >= OSPF_MAX_LSA)
 	{
-	  zlog_warn ("Pakcet [DD:RECV]: Unknown LS type %d.", lsah->type);
+	  zlog_warn ("Packet [DD:RECV]: Unknown LS type %d.", lsah->type);
 	  OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_SeqNumberMismatch);
 	  return;
 	}
@@ -1029,7 +1049,7 @@ ospf_db_desc_proc (struct stream *s, struct ospf_interface *oi,
 	{
 	  /* Received LSA is not recent. */
 	  if (IS_DEBUG_OSPF_EVENT)
-	    zlog_info ("Packet [DD:RECV]: LSA received Type %d, "
+	    zlog_debug ("Packet [DD:RECV]: LSA received Type %d, "
 		       "ID %s is not recent.", lsah->type, inet_ntoa (lsah->id));
 	  ospf_lsa_discard (new);
 	  continue;
@@ -1059,7 +1079,7 @@ ospf_db_desc_proc (struct stream *s, struct ospf_interface *oi,
 	  OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_ExchangeDone);
 	}
 
-      /* Send DD pakcet in reply. */
+      /* Send DD packet in reply. */
       ospf_db_desc_send (nbr);
     }
 
@@ -1080,7 +1100,7 @@ ospf_db_desc_is_dup (struct ospf_db_desc *dd, struct ospf_neighbor *nbr)
 }
 
 /* OSPF Database Description message read -- RFC2328 Section 10.6. */
-void
+static void
 ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
 	      struct stream *s, struct ospf_interface *oi, u_int16_t size)
 {
@@ -1103,7 +1123,9 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
   /* Check MTU. */
   if (ntohs (dd->mtu) > oi->ifp->mtu)
     {
-      zlog_warn ("Packet[DD]: MTU is larger than [%s]'s MTU", IF_NAME (oi));
+      zlog_warn ("Packet[DD]: Neighbor %s MTU %u is larger than [%s]'s MTU %u",
+		 inet_ntoa (nbr->router_id), ntohs (dd->mtu),
+		 IF_NAME (oi), oi->ifp->mtu);
       return;
     }
 
@@ -1125,7 +1147,7 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
        && (!CHECK_FLAG (dd->options, OSPF_OPTION_NP)) )
     {
       if (IS_DEBUG_OSPF_EVENT) 
-        zlog_notice ("Packet[DD]: Neighbour %s: Has NSSA capability, sends with N bit clear in DD options",
+        zlog_debug ("Packet[DD]: Neighbour %s: Has NSSA capability, sends with N bit clear in DD options",
                     inet_ntoa (nbr->router_id) );
       SET_FLAG (dd->options, OSPF_OPTION_NP);
     }
@@ -1160,7 +1182,8 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
     case NSM_Down:
     case NSM_Attempt:
     case NSM_TwoWay:
-      zlog_warn ("Packet[DD]: Neighbor state is %s, packet discarded.",
+      zlog_warn ("Packet[DD]: Neighbor %s state is %s, packet discarded.",
+		 inet_ntoa(nbr->router_id),
 		 LOOKUP (ospf_nsm_state_msg, nbr->state));
       break;
     case NSM_Init:
@@ -1178,14 +1201,16 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
 	  if (IPV4_ADDR_CMP (&nbr->router_id, &oi->ospf->router_id) > 0)
 	    {
 	      /* We're Slave---obey */
-	      zlog_warn ("Packet[DD]: Negotiation done (Slave).");
+	      zlog_info ("Packet[DD]: Neighbor %s Negotiation done (Slave).",
+	      		 inet_ntoa(nbr->router_id));
 	      nbr->dd_seqnum = ntohl (dd->dd_seqnum);
 	      nbr->dd_flags &= ~(OSPF_DD_FLAG_MS|OSPF_DD_FLAG_I); /* Reset I/MS */
 	    }
 	  else
 	    {
 	      /* We're Master, ignore the initial DBD from Slave */
-	      zlog_warn ("Packet[DD]: Initial DBD from Slave, ignoring.");
+	      zlog_warn ("Packet[DD]: Neighbor %s: Initial DBD from Slave, "
+	      		 "ignoring.", inet_ntoa(nbr->router_id));
 	      break;
 	    }
 	}
@@ -1194,12 +1219,14 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
 	       ntohl (dd->dd_seqnum) == nbr->dd_seqnum &&
 	       IPV4_ADDR_CMP (&nbr->router_id, &oi->ospf->router_id) < 0)
 	{
-	  zlog_warn ("Packet[DD]: Negotiation done (Master).");
+	  zlog_info ("Packet[DD]: Neighbor %s Negotiation done (Master).",
+		     inet_ntoa(nbr->router_id));
 	  nbr->dd_flags &= ~OSPF_DD_FLAG_I;
 	}
       else
 	{
-	  zlog_warn ("Packet[DD]: Negotiation fails.");
+	  zlog_warn ("Packet[DD]: Neighbor %s Negotiation fails.",
+		     inet_ntoa(nbr->router_id));
 	  break;
 	}
       
@@ -1210,7 +1237,7 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
       if (CHECK_FLAG (oi->ospf->config, OSPF_OPAQUE_CAPABLE))
         {
           if (IS_DEBUG_OSPF_EVENT)
-            zlog_info ("Neighbor[%s] is %sOpaque-capable.",
+            zlog_debug ("Neighbor[%s] is %sOpaque-capable.",
 		       inet_ntoa (nbr->router_id),
 		       CHECK_FLAG (nbr->options, OSPF_OPTION_O) ? "" : "NOT ");
 
@@ -1233,11 +1260,13 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
 	{
 	  if (IS_SET_DD_MS (nbr->dd_flags))
 	    /* Master: discard duplicated DD packet. */
-	    zlog_warn ("Packet[DD] (Master): packet duplicated.");
+	    zlog_warn ("Packet[DD] (Master): Neighbor %s packet duplicated.",
+		       inet_ntoa (nbr->router_id));
 	  else
 	    /* Slave: cause to retransmit the last Database Description. */
 	    {
-	      zlog_warn ("Packet[DD] [Slave]: packet duplicated.");
+	      zlog_warn ("Packet[DD] [Slave]: Neighbor %s packet duplicated.",
+			 inet_ntoa (nbr->router_id));
 	      ospf_db_desc_resend (nbr);
 	    }
 	  break;
@@ -1247,18 +1276,20 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
       /* Check Master/Slave bit mismatch */
       if (IS_SET_DD_MS (dd->flags) != IS_SET_DD_MS (nbr->last_recv.flags))
 	{
-	  zlog_warn ("Packet[DD]: MS-bit mismatch.");
+	  zlog_warn ("Packet[DD]: Neighbor %s MS-bit mismatch.",
+		     inet_ntoa(nbr->router_id));
 	  OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_SeqNumberMismatch);
 	  if (IS_DEBUG_OSPF_EVENT)
-	    zlog_info ("Packet[DD]: dd->flags=%d, nbr->dd_flags=%d",
-		       dd->flags, nbr->dd_flags);
+	    zlog_debug ("Packet[DD]: dd->flags=%d, nbr->dd_flags=%d",
+		        dd->flags, nbr->dd_flags);
 	  break;
 	}
 
       /* Check initialize bit is set. */
       if (IS_SET_DD_I (dd->flags))
 	{
-	  zlog_warn ("Packet[DD]: I-bit set.");
+	  zlog_warn ("Packet[DD]: Neighbor %s I-bit set.",
+		     inet_ntoa(nbr->router_id));
 	  OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_SeqNumberMismatch);
 	  break;
 	}
@@ -1270,7 +1301,8 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
 	  /* Save the new options for debugging */
 	  nbr->options = dd->options;
 #endif /* ORIGINAL_CODING */
-	  zlog_warn ("Packet[DD]: options mismatch.");
+	  zlog_warn ("Packet[DD]: Neighbor %s options mismatch.",
+		     inet_ntoa(nbr->router_id));
 	  OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_SeqNumberMismatch);
 	  break;
 	}
@@ -1281,7 +1313,8 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
 	  (!IS_SET_DD_MS (nbr->dd_flags) &&
 	   ntohl (dd->dd_seqnum) != nbr->dd_seqnum + 1))
 	{
-	  zlog_warn ("Pakcet[DD]: sequence number mismatch.");
+	  zlog_warn ("Packet[DD]: Neighbor %s sequence number mismatch.",
+		     inet_ntoa(nbr->router_id));
 	  OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_SeqNumberMismatch);
 	  break;
 	}
@@ -1296,7 +1329,8 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
 	  if (IS_SET_DD_MS (nbr->dd_flags))
 	    {
 	      /* Master should discard duplicate DD packet. */
-	      zlog_warn ("Pakcet[DD]: duplicated, packet discarded.");
+	      zlog_warn("Packet[DD]: Neighbor %s duplicated, packet discarded.",
+			inet_ntoa(nbr->router_id));
 	      break;
 	    }
 	  else
@@ -1324,7 +1358,8 @@ ospf_db_desc (struct ip *iph, struct ospf_header *ospfh,
       OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_SeqNumberMismatch);
       break;
     default:
-      zlog_warn ("Packet[DD]: NSM illegal status.");
+      zlog_warn ("Packet[DD]: Neighbor %s NSM illegal status %u.",
+		 inet_ntoa(nbr->router_id), nbr->state);
       break;
     }
 }
@@ -1360,7 +1395,9 @@ ospf_ls_req (struct ip *iph, struct ospf_header *ospfh,
       nbr->state != NSM_Loading &&
       nbr->state != NSM_Full)
     {
-      zlog_warn ("Link State Request: Neighbor state is %s, packet discarded.",
+      zlog_warn ("Link State Request received from %s: "
+      		 "Neighbor state is %s, packet discarded.",
+		 inet_ntoa (ospfh->router_id),
 		 LOOKUP (ospf_nsm_state_msg, nbr->state));
       return;
     }
@@ -1393,8 +1430,8 @@ ospf_ls_req (struct ip *iph, struct ospf_header *ospfh,
 	  return;
 	}
 
-      /* Packet overflows MTU size, send immediatly. */
-      if (length + ntohs (find->data->length) > OSPF_PACKET_MAX (oi))
+      /* Packet overflows MTU size, send immediately. */
+      if (length + ntohs (find->data->length) > ospf_packet_max (oi))
 	{
 	  if (oi->type == OSPF_IFTYPE_NBMA)
 	    ospf_ls_upd_send (nbr, ls_upd, OSPF_SEND_PACKET_DIRECT);
@@ -1503,7 +1540,7 @@ ospf_ls_upd_list_lsa (struct ospf_neighbor *nbr, struct stream *s,
 	      && nbr->oi->area->external_routing != OSPF_AREA_DEFAULT) 
             {
               if (IS_DEBUG_OSPF_EVENT)
-                zlog_info ("LSA[Type%d:%s]: We are a stub, don't take this LSA.", lsah->type, inet_ntoa (lsah->id));
+                zlog_debug ("LSA[Type%d:%s]: We are a stub, don't take this LSA.", lsah->type, inet_ntoa (lsah->id));
               continue;
             }
         }
@@ -1539,7 +1576,7 @@ ospf_ls_upd_list_lsa (struct ospf_neighbor *nbr, struct stream *s,
       memcpy (lsa->data, lsah, length);
 
       if (IS_DEBUG_OSPF_EVENT)
-	zlog_info("LSA[Type%d:%s]: %p new LSA created with Link State Update",
+	zlog_debug("LSA[Type%d:%s]: %p new LSA created with Link State Update",
 		  lsa->data->type, inet_ntoa (lsa->data->id), lsa);
       listnode_add (lsas, lsa);
     }
@@ -1592,8 +1629,10 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
   /* Check neighbor state. */
   if (nbr->state < NSM_Exchange)
     {
-      zlog_warn ("Link State Update: Neighbor[%s] state is less than Exchange",
-		 inet_ntoa (ospfh->router_id));
+      zlog_warn ("Link State Update: "
+      		 "Neighbor[%s] state %s is less than Exchange",
+		 inet_ntoa (ospfh->router_id),
+		 LOOKUP(ospf_nsm_state_msg, nbr->state));
       return;
     }
 
@@ -1624,7 +1663,7 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
 
 #define DISCARD_LSA(L,N) {\
         if (IS_DEBUG_OSPF_EVENT) \
-          zlog_info ("ospf_lsa_discard() in ospf_ls_upd() point %d: lsa %p Type-%d", N, lsa, (int) lsa->data->type); \
+          zlog_debug ("ospf_lsa_discard() in ospf_ls_upd() point %d: lsa %p Type-%d", N, lsa, (int) lsa->data->type); \
         ospf_lsa_discard (L); \
 	continue; }
 
@@ -1644,7 +1683,7 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
 	  char buf2[INET_ADDRSTRLEN];
 	  char buf3[INET_ADDRSTRLEN];
 
-	  zlog_info("LSA Type-%d from %s, ID: %s, ADV: %s",
+	  zlog_debug("LSA Type-%d from %s, ID: %s, ADV: %s",
 		  lsa->data->type,
 		  inet_ntop (AF_INET, &ospfh->router_id,
 			     buf1, INET_ADDRSTRLEN),
@@ -1677,7 +1716,7 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
 	  {
 	    DISCARD_LSA (lsa, 1);
 	    if (IS_DEBUG_OSPF_NSSA)
-	      zlog_info("Incoming External LSA Discarded: We are NSSA/STUB Area");
+	      zlog_debug("Incoming External LSA Discarded: We are NSSA/STUB Area");
 	  }
 
       if (lsa->data->type == OSPF_AS_NSSA_LSA)
@@ -1685,7 +1724,7 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
 	  {
 	    DISCARD_LSA (lsa,2);
 	    if (IS_DEBUG_OSPF_NSSA)
-	      zlog_info("Incoming NSSA LSA Discarded:  Not NSSA Area");
+	      zlog_debug("Incoming NSSA LSA Discarded:  Not NSSA Area");
 	  }
 
       /* Find the LSA in the current database. */
@@ -1705,7 +1744,8 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
 	  ospf_ls_ack_send (nbr, lsa);
 
 	  /* Discard LSA. */	  
-	  zlog_warn ("Link State Update: LS age is equal to MaxAge.");
+	  zlog_warn("Link State Update[%s]: LS age is equal to MaxAge.",
+		    dump_lsa_key(lsa));
           DISCARD_LSA (lsa, 3);
 	}
 
@@ -1745,7 +1785,7 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
           if (current == NULL)
             {
               if (IS_DEBUG_OSPF_EVENT)
-                zlog_info ("LSA[%s]: Previously originated Opaque-LSA, not found in the LSDB.", dump_lsa_key (lsa));
+                zlog_debug ("LSA[%s]: Previously originated Opaque-LSA, not found in the LSDB.", dump_lsa_key (lsa));
 
               SET_FLAG (lsa->flags, OSPF_LSA_SELF);
               listnode_add (mylsa_upds, ospf_lsa_dup  (lsa));
@@ -1779,7 +1819,7 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
             {
               ospf_lsa_flush_area(lsa,out_if->area);
               if(IS_DEBUG_OSPF_EVENT)
-                zlog_info ("ospf_lsa_discard() in ospf_ls_upd() point 9: lsa %p Type-%d",
+                zlog_debug ("ospf_lsa_discard() in ospf_ls_upd() point 9: lsa %p Type-%d",
                             lsa, (int) lsa->data->type);
               ospf_lsa_discard (lsa);
               Flag = 1;
@@ -1815,7 +1855,8 @@ ospf_ls_upd (struct ip *iph, struct ospf_header *ospfh,
       if (ospf_ls_request_lookup (nbr, lsa))
 	{
 	  OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_BadLSReq);
-	  zlog_warn ("LSA instance exists on Link state request list");
+	  zlog_warn("LSA[%s] instance exists on Link state request list",
+	  	    dump_lsa_key(lsa));
 
 	  /* Clean list of LSAs. */
           ospf_upd_list_clean (lsas);
@@ -1946,7 +1987,10 @@ ospf_ls_ack (struct ip *iph, struct ospf_header *ospfh,
 
   if (nbr->state < NSM_Exchange)
     {
-      zlog_warn ("Link State Acknowledgment: State is less than Exchange.");
+      zlog_warn ("Link State Acknowledgment: "
+      		 "Neighbor[%s] state %s is less than Exchange",
+		 inet_ntoa (ospfh->router_id),
+		 LOOKUP(ospf_nsm_state_msg, nbr->state));
       return;
     }
 
@@ -2007,12 +2051,8 @@ ospf_recv_packet (int fd, struct interface **ifp)
   struct stream *ibuf;
   unsigned int ifindex = 0;
   struct iovec iov;
-#if defined(CMSG_SPACE)
   /* Header and data both require alignment. */
   char buff [CMSG_SPACE(SOPT_SIZE_CMSG_IFINDEX_IPV4())];
-#else
-  char buff [sizeof (struct cmsghdr) + SOPT_SIZE_CMSG_IFINDEX_IPV4()];
-#endif
   struct msghdr msgh;
 
   memset (&msgh, 0, sizeof (struct msghdr));
@@ -2026,6 +2066,7 @@ ospf_recv_packet (int fd, struct interface **ifp)
   if (ret != sizeof (iph))
     {
       zlog_warn ("ospf_recv_packet packet smaller than ip header");
+      /* XXX: We peeked, and thus perhaps should discard this packet. */
       return NULL;
     }
   
@@ -2106,12 +2147,12 @@ ospf_associate_packet_vl (struct ospf *ospf, struct interface *ifp,
 	  IPV4_ADDR_SAME (&vl_data->vl_peer, &ospfh->router_id))
 	{
 	  if (IS_DEBUG_OSPF_EVENT)
-	    zlog_info ("associating packet with %s",
+	    zlog_debug ("associating packet with %s",
 		       IF_NAME (vl_data->vl_oi));
 	  if (! CHECK_FLAG (vl_data->vl_oi->ifp->flags, IFF_UP))
 	    {
 	      if (IS_DEBUG_OSPF_EVENT)
-		zlog_info ("This VL is not up yet, sorry");
+		zlog_debug ("This VL is not up yet, sorry");
 	      return NULL;
 	    }
 	  
@@ -2120,7 +2161,7 @@ ospf_associate_packet_vl (struct ospf *ospf, struct interface *ifp,
     }
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("couldn't find any VL to associate the packet with");
+    zlog_debug ("couldn't find any VL to associate the packet with");
   
   return NULL;
 }
@@ -2320,11 +2361,11 @@ ospf_read (struct thread *thread)
   iph = (struct ip *) STREAM_DATA (ibuf);
   sockopt_iphdrincl_swab_systoh (iph);
 
-  /* openbsd lacks IP_RECVIF */
-#if !(defined(IP_PKTINFO) || defined(IP_RECVIF))
   if (ifp == NULL)
+    /* Handle cases where the platform does not support retrieving the ifindex,
+       and also platforms (such as Solaris 8) that claim to support ifindex
+       retrieval but do not. */
     ifp = if_lookup_address (iph->ip_src);
-#endif /* !((defined(IP_PKTINFO) || defined(IP_RECVIF)) */
   
   if (ifp == NULL)
     {
@@ -2344,7 +2385,7 @@ ospf_read (struct thread *thread)
     {
       if (IS_DEBUG_OSPF_PACKET (0, RECV))
         {
-          zlog_info ("ospf_read[%s]: Dropping self-originated packet",
+          zlog_debug ("ospf_read[%s]: Dropping self-originated packet",
                      inet_ntoa (iph->ip_src));
         }
       stream_free (ibuf);
@@ -2389,6 +2430,14 @@ ospf_read (struct thread *thread)
       stream_free (ibuf);
       return 0;
     }
+  else if (oi->state == ISM_Down)
+    {
+      zlog_warn ("Ignoring packet from [%s] received on interface that is "
+      		 "down [%s]; interface flags are %s",
+                 inet_ntoa (iph->ip_src), ifp->name, if_flag_dump(ifp->flags));
+      stream_free (ibuf);
+      return 0;
+    }
 
   /*
    * If the received packet is destined for AllDRouters, the packet
@@ -2410,18 +2459,18 @@ ospf_read (struct thread *thread)
     {
       if (IS_DEBUG_OSPF_PACKET (ospfh->type - 1, DETAIL))
         {
-          zlog_info ("-----------------------------------------------------");
+          zlog_debug ("-----------------------------------------------------");
           ospf_packet_dump (ibuf);
         }
 
-      zlog_info ("%s received from [%s] via [%s]",
+      zlog_debug ("%s received from [%s] via [%s]",
                  ospf_packet_type_str[ospfh->type],
                  inet_ntoa (ospfh->router_id), IF_NAME (oi));
-      zlog_info (" src [%s],", inet_ntoa (iph->ip_src));
-      zlog_info (" dst [%s]", inet_ntoa (iph->ip_dst));
+      zlog_debug (" src [%s],", inet_ntoa (iph->ip_src));
+      zlog_debug (" dst [%s]", inet_ntoa (iph->ip_dst));
 
       if (IS_DEBUG_OSPF_PACKET (ospfh->type - 1, DETAIL))
-	zlog_info ("-----------------------------------------------------");
+	zlog_debug ("-----------------------------------------------------");
   }
 
   /* Some header verification. */
@@ -2430,7 +2479,7 @@ ospf_read (struct thread *thread)
     {
       if (IS_DEBUG_OSPF_PACKET (ospfh->type - 1, RECV))
         {
-          zlog_info ("ospf_read[%s/%s]: Header check failed, "
+          zlog_debug ("ospf_read[%s/%s]: Header check failed, "
                      "dropping.",
                      ospf_packet_type_str[ospfh->type],
                      inet_ntoa (iph->ip_src));
@@ -2579,7 +2628,7 @@ ospf_make_hello (struct ospf_interface *oi, struct stream *s)
   stream_putw (s, OSPF_IF_PARAM (oi, v_hello));
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("make_hello: options: %x, int: %s",
+    zlog_debug ("make_hello: options: %x, int: %s",
 	       OPTIONS(oi), IF_NAME (oi));
 
   /* Set Options. */
@@ -2714,7 +2763,7 @@ ospf_make_db_desc (struct ospf_interface *oi, struct ospf_neighbor *nbr,
 		u_int16_t ls_age;
 		
 		/* DD packet overflows interface MTU. */
-		if (length + OSPF_LSA_HEADER_SIZE > OSPF_PACKET_MAX (oi))
+		if (length + OSPF_LSA_HEADER_SIZE > ospf_packet_max (oi))
 		  break;
 		
 		/* Keep pointer to LS age. */
@@ -2749,7 +2798,7 @@ ospf_make_ls_req_func (struct stream *s, u_int16_t *length,
   oi = nbr->oi;
 
   /* LS Request packet overflows interface MTU. */
-  if (*length + delta > OSPF_PACKET_MAX(oi))
+  if (*length + delta > ospf_packet_max(oi))
     return 0;
 
   stream_putl (s, lsa->data->type);
@@ -2806,30 +2855,34 @@ ospf_make_ls_upd (struct ospf_interface *oi, struct list *update, struct stream
   struct ospf_lsa *lsa;
   struct listnode *node;
   u_int16_t length = OSPF_LS_UPD_MIN_SIZE;
+  unsigned int size_noauth;
   unsigned long delta = stream_get_putp (s);
   unsigned long pp;
   int count = 0;
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("ospf_make_ls_upd: Start");
+    zlog_debug ("ospf_make_ls_upd: Start");
 
   pp = stream_get_putp (s);
   ospf_output_forward (s, OSPF_LS_UPD_MIN_SIZE);
 
+  /* Calculate amount of packet usable for data. */
+  size_noauth = stream_get_size(s) - ospf_packet_authspace(oi);
+
   while ((node = listhead (update)) != NULL)
     {
       struct lsa_header *lsah;
       u_int16_t ls_age;
 
       if (IS_DEBUG_OSPF_EVENT)
-        zlog_info ("ospf_make_ls_upd: List Iteration");
+        zlog_debug ("ospf_make_ls_upd: List Iteration");
 
       lsa = getdata (node);
       assert (lsa);
       assert (lsa->data);
 
       /* Will it fit? */
-      if (length + delta + ntohs (lsa->data->length) > stream_get_size (s))
+      if (length + delta + ntohs (lsa->data->length) > size_noauth)
         break;
 
       /* Keep pointer to LS age. */
@@ -2858,7 +2911,7 @@ ospf_make_ls_upd (struct ospf_interface *oi, struct list *update, struct stream
   stream_set_putp (s, s->endp);
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("ospf_make_ls_upd: Stop");
+    zlog_debug ("ospf_make_ls_upd: Stop");
   return length;
 }
 
@@ -2878,7 +2931,7 @@ ospf_make_ls_ack (struct ospf_interface *oi, struct list *ack, struct stream *s)
       lsa = getdata (node);
       assert (lsa);
       
-      if (length + delta > OSPF_PACKET_MAX (oi))
+      if (length + delta > ospf_packet_max (oi))
 	break;
       
       stream_put (s, lsa->data, OSPF_LSA_HEADER_SIZE);
@@ -2967,7 +3020,7 @@ ospf_poll_timer (struct thread *thread)
   nbr_nbma->t_poll = NULL;
 
   if (IS_DEBUG_OSPF (nsm, NSM_TIMERS))
-    zlog (NULL, LOG_INFO, "NSM[%s:%s]: Timer (Poll timer expire)",
+    zlog (NULL, LOG_DEBUG, "NSM[%s:%s]: Timer (Poll timer expire)",
     IF_NAME (nbr_nbma->oi), inet_ntoa (nbr_nbma->addr));
 
   ospf_poll_send (nbr_nbma);
@@ -2991,7 +3044,7 @@ ospf_hello_reply_timer (struct thread *thread)
   assert (nbr->oi);
 
   if (IS_DEBUG_OSPF (nsm, NSM_TIMERS))
-    zlog (NULL, LOG_INFO, "NSM[%s:%s]: Timer (hello-reply timer expire)",
+    zlog (NULL, LOG_DEBUG, "NSM[%s:%s]: Timer (hello-reply timer expire)",
 	  IF_NAME (nbr->oi), inet_ntoa (nbr->router_id));
 
   ospf_hello_send_sub (nbr->oi, &nbr->address.u.prefix4);
@@ -3227,7 +3280,7 @@ ospf_ls_upd_packet_new (struct list *update, struct ospf_interface *oi)
         }
 
       if (IS_DEBUG_OSPF_PACKET (0, SEND))
-        zlog_warn ("ospf_ls_upd_packet_new: oversized LSA id:%s,"
+        zlog_debug ("ospf_ls_upd_packet_new: oversized LSA id:%s,"
                    " %d bytes originated by %s, will be fragmented!",
                    inet_ntoa (lsa->data->id),
                    ntohs (lsa->data->length),
@@ -3243,6 +3296,7 @@ ospf_ls_upd_packet_new (struct list *update, struct ospf_interface *oi)
   else
     size = oi->ifp->mtu;
 
+  /* XXX Should this be - sizeof(struct ip)?? -gdt */
   if (size > OSPF_MAX_PACKET_SIZE)
     {
       zlog_warn ("ospf_ls_upd_packet_new: oversized LSA id:%s too big,"
@@ -3265,7 +3319,7 @@ ospf_ls_upd_queue_send (struct ospf_interface *oi, struct list *update,
   u_int16_t length = OSPF_HEADER_SIZE;
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("listcount = %d, dst %s", listcount (update), inet_ntoa(addr));
+    zlog_debug ("listcount = %d, dst %s", listcount (update), inet_ntoa(addr));
   
   op = ospf_ls_upd_packet_new (update, oi);
 
@@ -3305,7 +3359,7 @@ ospf_ls_upd_send_queue_event (struct thread *thread)
   oi->t_ls_upd_event = NULL;
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("ospf_ls_upd_send_queue start");
+    zlog_debug ("ospf_ls_upd_send_queue start");
 
   for (rn = route_top (oi->ls_upd_queue); rn; rn = rnext)
     {
@@ -3332,14 +3386,14 @@ ospf_ls_upd_send_queue_event (struct thread *thread)
   if (again != 0)
     {
       if (IS_DEBUG_OSPF_EVENT)
-        zlog_info ("ospf_ls_upd_send_queue: update lists not cleared,"
+        zlog_debug ("ospf_ls_upd_send_queue: update lists not cleared,"
                    " %d nodes to try again, raising new event", again);
       oi->t_ls_upd_event = 
         thread_add_event (master, ospf_ls_upd_send_queue_event, oi, 0);
     }
 
   if (IS_DEBUG_OSPF_EVENT)
-    zlog_info ("ospf_ls_upd_send_queue stop");
+    zlog_debug ("ospf_ls_upd_send_queue stop");
   
   return 0;
 }
