@@ -1251,7 +1251,7 @@ zebra_client_read (struct thread *thread)
   if (nbyte <= 0) 
     {
       if (IS_ZEBRA_DEBUG_EVENT)
-	zlog_info ("connection closed socket [%d]", sock);
+	zlog_debug ("connection closed socket [%d]", sock);
       zebra_client_close (client);
       return -1;
     }
@@ -1261,7 +1261,7 @@ zebra_client_read (struct thread *thread)
   if (length < 3) 
     {
       if (IS_ZEBRA_DEBUG_EVENT)
-	zlog_info ("length %d is less than 3 ", length);
+	zlog_debug ("length %d is less than 3 ", length);
       zebra_client_close (client);
       return -1;
     }
@@ -1275,7 +1275,7 @@ zebra_client_read (struct thread *thread)
       if (nbyte <= 0) 
 	{
 	  if (IS_ZEBRA_DEBUG_EVENT)
-	    zlog_info ("connection closed [%d] when reading zebra data", sock);
+	    zlog_debug ("connection closed [%d] when reading zebra data", sock);
 	  zebra_client_close (client);
 	  return -1;
 	}
@@ -1283,10 +1283,10 @@ zebra_client_read (struct thread *thread)
 
   /* Debug packet information. */
   if (IS_ZEBRA_DEBUG_EVENT)
-    zlog_info ("zebra message comes from socket [%d]", sock);
+    zlog_debug ("zebra message comes from socket [%d]", sock);
 
   if (IS_ZEBRA_DEBUG_PACKET && IS_ZEBRA_DEBUG_RECV)
-    zlog_info ("zebra message received [%s] %d", 
+    zlog_debug ("zebra message received [%s] %d", 
 	       zebra_command_str[command], length);
 
   switch (command) 
@@ -1369,15 +1369,19 @@ zebra_accept (struct thread *thread)
 
   if (client_sock < 0)
     {
-      zlog_warn ("Can't accept zebra socket: %s", strerror (errno));
+      zlog_warn ("Can't accept zebra socket: %s", safe_strerror (errno));
       return -1;
     }
 
   /* Make client socket non-blocking.  */
-
+  /* XXX: We dont requeue failed writes, so this leads to inconsistencies.
+   * for now socket must remain blocking, regardless of risk of deadlocks.
+   */
+  /*
   val = fcntl (client_sock, F_GETFL, 0);
   fcntl (client_sock, F_SETFL, (val | O_NONBLOCK));
-
+  */
+  
   /* Create new zebra client. */
   zebra_client_create (client_sock);
 
@@ -1400,7 +1404,7 @@ zebra_serv ()
 
   if (accept_sock < 0) 
     {
-      zlog_warn ("Can't bind to socket: %s", strerror (errno));
+      zlog_warn ("Can't bind to socket: %s", safe_strerror (errno));
       zlog_warn ("zebra can't provice full functionality due to above error");
       return;
     }
@@ -1423,7 +1427,7 @@ zebra_serv ()
 	       sizeof (struct sockaddr_in));
   if (ret < 0)
     {
-      zlog_warn ("Can't bind to socket: %s", strerror (errno));
+      zlog_warn ("Can't bind to socket: %s", safe_strerror (errno));
       zlog_warn ("zebra can't provice full functionality due to above error");
       close (accept_sock);      /* Avoid sd leak. */
       return;
@@ -1435,7 +1439,7 @@ zebra_serv ()
   ret = listen (accept_sock, 1);
   if (ret < 0)
     {
-      zlog_warn ("Can't listen to socket: %s", strerror (errno));
+      zlog_warn ("Can't listen to socket: %s", safe_strerror (errno));
       zlog_warn ("zebra can't provice full functionality due to above error");
       close (accept_sock);	/* Avoid sd leak. */
       return;
